/*****************************************************************/
/*  ELEC 999                                                     */
/*  University of Nebraska-Lincoln                               */
/*  Department of Electrical Engineering                         */
/*  David J. Russell                                             */
/*  3-5-10                                                       */
/*                                                               */
/* 8/24/10  - speed up by using a modified grammar distance in   */
/*             CalculateDistance, version 1.1                    */
/*****************************************************************/

#define _GRAM

/*********************************************************************************/
/* Included Header Files */
/*********************************************************************************/
#include "Globals.h"
#include "StringADT.h"
#include "SuffixTreeADT.h"
#include "Gram.h"

/*********************************************************************************/
/* Private Module Constants */
/*********************************************************************************/

/*********************************************************************************/
/* Private Type Definitions */
/*********************************************************************************/

/*********************************************************************************/
/* Private Module Function Prototypes */
/*********************************************************************************/

static FLOAT GRAM_LZComparePart(STADT_TREE_TYPE *Sequence1, 
                                 FLOAT DictionarySize1,
                                 STADT_TREE_TYPE *Sequence2);
static void GRAM_CreateLZDictionary(STADT_TREE_TYPE *Sequence, 
                                    FLOAT *DictionarySize);
static BOOLEAN GRAM_LZAdd(BYTE Character, 
                           FLOAT *CurrentDictionarySize, 
                           STRINGADT_STRING_TYPE *CurrentFragment,
                           STRINGADT_STRING_TYPE *CurrentSequence,
                           ULONG *CurrentPositionIndex);
static BOOLEAN GRAM_LZScan(STADT_TREE_TYPE *Sequence1,
                           STADT_TREE_TYPE *Sequence2,
                           STRINGADT_STRING_TYPE *CurrentFragment,
                           ULONG Index);

/*********************************************************************************/
/* Private Module Variables */
/*********************************************************************************/

/*********************************************************************************/
/* Public and Private Module Functions */
/*********************************************************************************/

FLOAT GRAM_CreateIndividualDictionary(STADT_TREE_TYPE *Sequence)
{
   FLOAT DictionarySize = 0;

   GRAM_CreateLZDictionary(Sequence, &DictionarySize);

   return (DictionarySize);
}

/*********************************************************************************/

FLOAT GRAM_CalculateDistance(STADT_TREE_TYPE *Sequence1, 
                              FLOAT DictionarySize1,
                              STADT_TREE_TYPE *Sequence2,
                              FLOAT DictionarySize2)
{
   FLOAT ComplexitySQ;
   FLOAT ComplexityQS;
   FLOAT ComplexityS;
   FLOAT ComplexityQ;
   FLOAT Distance;

   ComplexityS = DictionarySize1;
   ComplexityQ = DictionarySize2;

   if (Sequence1->Length > Sequence2->Length)
   {
      ComplexitySQ = GRAM_LZComparePart(Sequence1, DictionarySize1, Sequence2);

      Distance = (((ComplexitySQ - ComplexityS) / ComplexityS) / ((FLOAT) Sequence2->Length / (FLOAT) Sequence1->Length));
   }
   else
   {
      ComplexityQS = GRAM_LZComparePart(Sequence2, DictionarySize2, Sequence1);

      Distance = (((ComplexityQS - ComplexityQ) / ComplexityQ) / ((FLOAT) Sequence1->Length / (FLOAT) Sequence2->Length));
   }
   
   return (Distance);
}

/*********************************************************************************/

void GRAM_CreateLZDictionary(STADT_TREE_TYPE *Sequence, 
                              FLOAT *DictionarySize)
{
   ULONG i;
   STRINGADT_STRING_TYPE BuildSequence;
   STRINGADT_STRING_TYPE Fragment;
   ULONG PositionIndex;
   
   STRINGADT_InitializeString(&BuildSequence);
   STRINGADT_InitializeString(&Fragment);
   PositionIndex = 0;
   
   for (i = 0; i < (Sequence->Length - 1); i++)
   {
      Sequence->Reproducible[i] = GRAM_LZAdd(Sequence->String[i],
                                             DictionarySize,
                                             &Fragment,
                                             &BuildSequence,
                                             &PositionIndex);
   }

   STRINGADT_CleanUpString(&BuildSequence);
   STRINGADT_CleanUpString(&Fragment);
}

/*********************************************************************************/

FLOAT GRAM_LZComparePart(STADT_TREE_TYPE *Sequence1, 
                           FLOAT DictionarySize1,
                           STADT_TREE_TYPE *Sequence2)
{
   ULONG i;
   FLOAT DictionarySize;
   STRINGADT_STRING_TYPE Fragment;

   STRINGADT_InitializeString(&Fragment);
   DictionarySize = DictionarySize1;

   for (i = 0; i < Sequence2->Length; i++)
   {
      if (GRAM_LZScan(Sequence1, Sequence2, &Fragment, i) == FALSE)
      {
         DictionarySize++;
      }
   }

   STRINGADT_CleanUpString(&Fragment);

   return (DictionarySize);
}

/*********************************************************************************/

BOOLEAN GRAM_LZAdd(BYTE Character, 
                  FLOAT *CurrentDictionarySize, 
                  STRINGADT_STRING_TYPE *CurrentFragment,
                  STRINGADT_STRING_TYPE *CurrentSequence,
                  ULONG *CurrentPositionIndex)
{
   BOOLEAN Reproducible = FALSE;

   STRINGADT_AppendStringFromChar(CurrentFragment, Character, "GRAM_LZAdd");

   if (*CurrentDictionarySize == 0)
   {
      *CurrentDictionarySize = (*CurrentDictionarySize) + 1;      
      STRINGADT_CleanUpString(CurrentFragment);
      STRINGADT_InitializeString(CurrentFragment);
      *CurrentPositionIndex = 0;
   }
   else
   {
      Reproducible = STRINGADT_FindString(CurrentSequence, CurrentFragment, CurrentPositionIndex);
      if (Reproducible == FALSE)
      {
         *CurrentDictionarySize = (*CurrentDictionarySize) + 1;      
         STRINGADT_CleanUpString(CurrentFragment);
         STRINGADT_InitializeString(CurrentFragment);
         *CurrentPositionIndex = 0;
      }
   }
   
   STRINGADT_AppendStringFromChar(CurrentSequence, Character, "GRAM_LZAdd");

   return (Reproducible);
}

/*********************************************************************************/

BOOLEAN GRAM_LZScan(STADT_TREE_TYPE *Sequence1,
                     STADT_TREE_TYPE *Sequence2,
                     STRINGADT_STRING_TYPE *CurrentFragment,
                     ULONG Index)
{
   BOOLEAN Reproducible = TRUE;

   STRINGADT_AppendStringFromChar(CurrentFragment, Sequence2->String[Index], "GRAM_LZScan");

   if (Sequence2->Reproducible[Index] == FALSE)
   {
      Reproducible = STADT_FindSubstring (Sequence1, CurrentFragment->Characters, CurrentFragment->Length);

      STRINGADT_CleanUpString(CurrentFragment);
      STRINGADT_InitializeString(CurrentFragment);
   }
   
   return (Reproducible);
}
