/*****************************************************************/
/*  ELEC 999                                                     */
/*  University of Nebraska-Lincoln                               */
/*  Department of Electrical Engineering                         */
/*  David J. Russell                                             */
/*  3-5-10                                                       */
/*****************************************************************/

#define _MATRIXDOUBLEADT

/*********************************************************************************/
/* Included Header Files */
/*********************************************************************************/
#include "Globals.h"
#include "MatrixDoubleADT.h"

/*********************************************************************************/
/* Private Module Constants */
/*********************************************************************************/

/*********************************************************************************/
/* Private Type Definitions */
/*********************************************************************************/

/*********************************************************************************/
/* Private Module Function Prototypes */
/*********************************************************************************/
static void MATRIXDOUBLEADT_DefineMatrix(MATRIXDOUBLEADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols);

/*********************************************************************************/
/* Private Module Variables */
/*********************************************************************************/

/*********************************************************************************/
/* Public and Private Module Functions */
/*********************************************************************************/

void MATRIXDOUBLEADT_InitializeMatrix(MATRIXDOUBLEADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols)
{
   Matrix->NumRows = 0;
   Matrix->NumCols = 0;
   Matrix->Rows = NULL;

   MATRIXDOUBLEADT_DefineMatrix(Matrix, Rows, Cols);
}

/*********************************************************************************/

void MATRIXDOUBLEADT_DefineMatrix(MATRIXDOUBLEADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols)
{
   ULONG i;
   ULONG j;
   
   Matrix->NumRows = Rows;
   Matrix->NumCols = Cols;
   Matrix->Rows = (MATRIXDOUBLEADT_ROW_TYPE *) malloc (sizeof(MATRIXDOUBLEADT_ROW_TYPE) * Rows);
   if (Matrix->Rows == NULL)
   {
      printf("Error: malloc failed in MATRIXDOUBLEADT_InitializeMatrix.\n");
      exit(1);
   }
   else
   {
      for (i = 0; i < Rows; i++)
      {
         Matrix->Rows[i].Cols = (DOUBLE *) malloc (sizeof(DOUBLE) * Cols);
         if (Matrix->Rows[i].Cols == NULL)
         {
            printf("Error: malloc failed in MATRIXDOUBLEADT_InitializeMatrix.\n");
            exit(1);
         }
         else
         {
            /*
               Warning: memset does not necessarily work for floating point types.
            */
            for (j = 0; j < Cols; j++)
            {
               Matrix->Rows[i].Cols[j] = 0;
            }
         }
      }
   }
}

/*********************************************************************************/

void MATRIXDOUBLEADT_RedefineMatrix(MATRIXDOUBLEADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols)
{
   ULONG i;
   ULONG j;

   if ((Matrix->NumRows == 0) || (Matrix->NumCols == 0))
   {
      MATRIXDOUBLEADT_DefineMatrix(Matrix, Rows, Cols);
   }
   else
   {
      if (Rows > Matrix->NumRows)
      {
         Matrix->Rows = (MATRIXDOUBLEADT_ROW_TYPE *) realloc (Matrix->Rows, sizeof(MATRIXDOUBLEADT_ROW_TYPE) * Rows);
         if (Matrix->Rows == NULL)
         {
            printf("Error: realloc failed in MATRIXDOUBLEADT_RedefineMatrix.\n");
            exit(1);
         }
         else
         {
            for (i = Matrix->NumRows; i < Rows; i++)
            {
               Matrix->Rows[i].Cols = (DOUBLE *) malloc (sizeof(DOUBLE) * Matrix->NumCols);
               if (Matrix->Rows[i].Cols == NULL)
               {
                  printf("Error: malloc failed in MATRIXDOUBLEADT_RedefineMatrix.\n");
                  exit(1);
               }
               else
               {
                  /*
                     Warning: memset does not necessarily work for floating point types.
                  */
                  for (j = 0; j < Matrix->NumCols; j++)
                  {
                     Matrix->Rows[i].Cols[j] = 0;
                  }
               }
            }
         }

         Matrix->NumRows = Rows;
      }

      if (Cols > Matrix->NumCols)
      {
         for (i = 0; i < Matrix->NumRows; i++)
         {
            Matrix->Rows[i].Cols = (DOUBLE *) realloc (Matrix->Rows[i].Cols, sizeof(DOUBLE) * Cols);
            if (Matrix->Rows[i].Cols == NULL)
            {
               printf("Error: realloc failed in MATRIXDOUBLEADT_RedefineMatrix.\n");
               exit(1);
            }
            else
            {
               /*
                  Warning: memset does not necessarily work for floating point types.
               */
               for (j = Matrix->NumCols; j < Cols; j++)
               {
                  Matrix->Rows[i].Cols[j] = 0;
               }
            }
         }

         Matrix->NumCols = Cols;
      }
   }
}

/*********************************************************************************/

void MATRIXDOUBLEADT_CleanUpMatrix(MATRIXDOUBLEADT_MATRIX_TYPE *Matrix)
{
   ULONG i;

   for (i = 0; i < Matrix->NumRows; i++)
   {
      if (Matrix->Rows[i].Cols != NULL)
      {
         free(Matrix->Rows[i].Cols);
         Matrix->Rows[i].Cols = NULL;
      }
   }

   if (Matrix->Rows != NULL)
   {
      free(Matrix->Rows);
      Matrix->Rows = NULL;
   }

   Matrix->NumRows = 0;
   Matrix->NumCols = 0;
}
