/*****************************************************************/
/*  ELEC 999                                                     */
/*  University of Nebraska-Lincoln                               */
/*  Department of Electrical Engineering                         */
/*  David J. Russell                                             */
/*  3-5-10                                                       */
/*****************************************************************/

#define _MATRIXULONGADT

/*********************************************************************************/
/* Included Header Files */
/*********************************************************************************/
#include "Globals.h"
#include "MatrixUlongADT.h"

/*********************************************************************************/
/* Private Module Constants */
/*********************************************************************************/

/*********************************************************************************/
/* Private Type Definitions */
/*********************************************************************************/

/*********************************************************************************/
/* Private Module Function Prototypes */
/*********************************************************************************/
static void MATRIXULONGADT_DefineMatrix(MATRIXULONGADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols);

/*********************************************************************************/
/* Private Module Variables */
/*********************************************************************************/

/*********************************************************************************/
/* Public and Private Module Functions */
/*********************************************************************************/

void MATRIXULONGADT_InitializeMatrix(MATRIXULONGADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols)
{
   Matrix->NumRows = 0;
   Matrix->NumCols = 0;
   Matrix->Rows = NULL;

   MATRIXULONGADT_DefineMatrix(Matrix, Rows, Cols);
}

/*********************************************************************************/

void MATRIXULONGADT_DefineMatrix(MATRIXULONGADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols)
{
   ULONG i;
   
   Matrix->NumRows = Rows;
   Matrix->NumCols = Cols;
   Matrix->Rows = (MATRIXULONGADT_ROW_TYPE *) malloc (sizeof(MATRIXULONGADT_ROW_TYPE) * Rows);
   if (Matrix->Rows == NULL)
   {
      printf("Error: malloc failed in MATRIXULONGADT_DefineMatrix.\n");
      exit(1);
   }
   else
   {
      for (i = 0; i < Rows; i++)
      {
         Matrix->Rows[i].Cols = (ULONG *) malloc (sizeof(ULONG) * Cols);
         if (Matrix->Rows[i].Cols == NULL)
         {
            printf("Error: malloc failed in MATRIXULONGADT_DefineMatrix.\n");
            exit(1);
         }
         else
         {
            memset(Matrix->Rows[i].Cols, 0, sizeof(ULONG) * Cols);
         }
      }
   }
}

/*********************************************************************************/

void MATRIXULONGADT_RedefineMatrix(MATRIXULONGADT_MATRIX_TYPE *Matrix, ULONG Rows, ULONG Cols)
{
   ULONG i;

   if ((Matrix->NumRows == 0) || (Matrix->NumCols == 0))
   {
      MATRIXULONGADT_DefineMatrix(Matrix, Rows, Cols);
   }
   else
   {
      if (Rows > Matrix->NumRows)
      {
         Matrix->Rows = (MATRIXULONGADT_ROW_TYPE *) realloc (Matrix->Rows, sizeof(MATRIXULONGADT_ROW_TYPE) * Rows);
         if (Matrix->Rows == NULL)
         {
            printf("Error: realloc failed in MATRIXULONGADT_RedefineMatrix.\n");
            exit(1);
         }
         else
         {
            for (i = Matrix->NumRows; i < Rows; i++)
            {
               Matrix->Rows[i].Cols = (ULONG *) malloc (sizeof(ULONG) * Matrix->NumCols);
               if (Matrix->Rows[i].Cols == NULL)
               {
                  printf("Error: malloc failed in MATRIXULONGADT_RedefineMatrix.\n");
                  exit(1);
               }
               else
               {
                  memset(Matrix->Rows[i].Cols, 0, sizeof(ULONG) * Matrix->NumCols);
               }
            }
         }

         Matrix->NumRows = Rows;
      }

      if (Cols > Matrix->NumCols)
      {
         for (i = 0; i < Matrix->NumRows; i++)
         {
            Matrix->Rows[i].Cols = (ULONG *) realloc (Matrix->Rows[i].Cols, sizeof(ULONG) * Cols);
            if (Matrix->Rows[i].Cols == NULL)
            {
               printf("Error: realloc failed in MATRIXULONGADT_RedefineMatrix.\n");
               exit(1);
            }
            else
            {
               memset(&(Matrix->Rows[i].Cols[Matrix->NumCols]), 0, sizeof(ULONG) * (Cols - Matrix->NumCols));
            }
         }

         Matrix->NumCols = Cols;
      }
   }
}

/*********************************************************************************/

void MATRIXULONGADT_CleanUpMatrix(MATRIXULONGADT_MATRIX_TYPE *Matrix)
{
   ULONG i;

   for (i = 0; i < Matrix->NumRows; i++)
   {
      if (Matrix->Rows[i].Cols != NULL)
      {
         free(Matrix->Rows[i].Cols);
         Matrix->Rows[i].Cols = NULL;
      }
   }

   if (Matrix->Rows != NULL)
   {
      free(Matrix->Rows);
      Matrix->Rows = NULL;
   }

   Matrix->NumRows = 0;
   Matrix->NumCols = 0;
}
