/*****************************************************************/
/*  ELEC 999                                                     */
/*  University of Nebraska-Lincoln                               */
/*  Department of Electrical Engineering                         */
/*  David J. Russell                                             */
/*  3-5-10                                                       */
/*****************************************************************/

#define _STRINGLISTADT

/*********************************************************************************/
/* Included Header Files */
/*********************************************************************************/
#include "Globals.h"
#include "StringADT.h"
#include "StringListADT.h"

/*********************************************************************************/
/* Private Module Constants */
/*********************************************************************************/

/*********************************************************************************/
/* Private Type Definitions */
/*********************************************************************************/
#define STRINGLISTADT_CHUNK_SIZE 64

/*********************************************************************************/
/* Private Module Function Prototypes */
/*********************************************************************************/

/*********************************************************************************/
/* Private Module Variables */
/*********************************************************************************/

/*********************************************************************************/
/* Public and Private Module Functions */
/*********************************************************************************/

void STRINGLISTADT_InitializeStringList(STRINGLISTADT_STRINGLIST_TYPE *StringList)
{
   StringList->NumberOfStringsAllocated = 0;
   StringList->NumberOfStrings = 0;
   StringList->Strings = NULL;
}

/*********************************************************************************/

void STRINGLISTADT_CleanUpStringList(STRINGLISTADT_STRINGLIST_TYPE *StringList)
{
   ULONG i;

   for (i = 0; i < StringList->NumberOfStrings; i++)
   {
      STRINGADT_CleanUpString (&(StringList->Strings[i]));
   }

   if (StringList->Strings != NULL)
   {
      free(StringList->Strings);
   }
   
   StringList->NumberOfStringsAllocated = 0;
   StringList->NumberOfStrings = 0;
   StringList->Strings = NULL;
}

/*********************************************************************************/

void STRINGLISTADT_AppendStringToStringList(STRINGLISTADT_STRINGLIST_TYPE *StringList, 
                                             STRINGADT_STRING_TYPE *String)
{
   if (StringList->NumberOfStrings == 0)
   {
      StringList->Strings = (STRINGADT_STRING_TYPE *) malloc(sizeof(STRINGADT_STRING_TYPE) * STRINGLISTADT_CHUNK_SIZE);
      if (StringList->Strings == NULL)
      {
         printf("Error: malloc failed in STRINGLISTADT_AppendStringToStringList.\n");
         exit(1);
      }
      StringList->NumberOfStringsAllocated = STRINGLISTADT_CHUNK_SIZE;
   }
   else if (StringList->NumberOfStrings >= StringList->NumberOfStringsAllocated)
   {
      StringList->Strings = (STRINGADT_STRING_TYPE *) realloc(StringList->Strings, sizeof(STRINGADT_STRING_TYPE) * (StringList->NumberOfStringsAllocated + STRINGLISTADT_CHUNK_SIZE));
      if (StringList->Strings == NULL)
      {
         printf("Error: realloc failed in STRINGLISTADT_AppendStringToStringList.\n");
         exit(1);
      }
      StringList->NumberOfStringsAllocated += STRINGLISTADT_CHUNK_SIZE;
   }

   StringList->NumberOfStrings += 1;
   STRINGADT_InitializeString(&(StringList->Strings[StringList->NumberOfStrings - 1]));
   STRINGADT_AppendStringFromString (&(StringList->Strings[StringList->NumberOfStrings - 1]), 
                                       String,
                                       "STRINGLISTADT_AppendStringToStringList");
}
